package Client;

use strict;
use warnings;

use LWP::UserAgent::Retrying;

use Configurator;
use ClientManagers;
use Task;

use fields qw(STATE CONFIGURATOR TASK PORTIONS_EXECUTED);

# Client state constants
use constant {
  INITIAL       => 1, # Initial state
  GOT_TASK_DESC => 2, # Got task description
  TASK_READY    => 3, # All files for task are here
  GOT_PORTION   => 4, # Got portion, processing it
  PORTION_DONE  => 5  # Finished processing portion, sending
};

my @state_processors;
$state_processors[INITIAL]       = \&process_initial;
$state_processors[GOT_TASK_DESC] = \&process_got_task_desc;
$state_processors[TASK_READY]    = \&process_task_ready;
$state_processors[GOT_PORTION]   = \&process_got_portion;
$state_processors[PORTION_DONE]  = \&process_portion_done;

sub new
{
  my $self = shift;
  $self = fields::new($self) unless (ref $self);

  my %args = @_;

  $self->{STATE} = INITIAL;
  $self->{CONFIGURATOR} = $args{CONFIGURATOR}
    or die "No configurator";
  $self->{PORTIONS_EXECUTED} = 0;

  return $self;
}

sub state
{
  my $self = shift;
  if (@_) {$self->{STATE} = shift;}
  return $self->{STATE};
}

sub configurator
{
  my $self = shift;
  return $self->{CONFIGURATOR};
}

sub portions_executed
{
  my $self = shift;
  if (@_) {$self->{PORTIONS_EXECUTED} = shift;}
  return $self->{PORTIONS_EXECUTED};
}

sub task
{
  my $self = shift;
  return $self->{TASK};
}

sub run
{
  my $self = shift;

  while(1) {
    my $state = $self->state();
    my $processor = $state_processors[$state];
    if (not defined($processor)) {
      die "Invalid client state detected";
    }
    $self->$processor;
  }
}

sub process_initial
{
  my $self = shift;

  my $conf = $self->configurator();
  
  my $climan = ClientManagers->new('fallback', $conf->climan_urls());
  if (defined $conf->ca_path){
    $ENV{HTTPS_CA_DIR}  = $conf->ca_path();# despite new LWP with explicit SSL configuration,
    # removal of this variable causes miserable certificate verification failure
    # should be assigned BEFORE $ua is created
  }
  if ((defined $conf->cert_file) && (defined $conf->key_file) && (defined $conf->ca_path)){
    $ENV{HTTPS_CERT_FILE} = $conf->cert_file();
    $ENV{HTTPS_KEY_FILE}  = $conf->key_file();  
  }
  if ($conf->debug)
  {
    $ENV{HTTPS_DEBUG} = 1;
  }

  my $ua = LWP::UserAgent::Retrying->new(
    agent => $conf->ua_string(),
    requests_redirectable => ['GET', 'POST']  
  );
  if($conf->http_proxy()) {
    $ua->proxy('http', $conf->http_proxy());
  }
  if (defined $ua->ssl_opts) #new version of LWP on IO::Socket::SSL - to be configured by hand: some of $env are somehow ignored under mswin32
  {
    if ((defined $conf->ca_path))
    {
      $ua->ssl_opts(
      SSL_ca_path => $conf->ca_path(),
      SSL_verify_mode => 0x03
      );
    }
    if ((defined $conf->cert_file) && (defined $conf->key_file) && (defined $conf->ca_path))
    {      
      $ua->ssl_opts(
      SSL_ca_path => $conf->ca_path(),      
      SSL_key_file => $conf->key_file(),
      SSL_cert_file => $conf->cert_file(),      
      SSL_verify_mode => 0x03
      );
    }    
  } 

  warn "Initializing.\n";
  $self->{TASK} = Task->new($self, $climan, $ua)
    or die "Cannot fetch task description";

  $self->state(GOT_TASK_DESC);
}

sub process_got_task_desc
{
  my $self = shift;

  warn "Fetching working files.\n";
  $self->task()->fetch_files();
  warn "Initializing client part.\n";
  $self->task()->init_clipart();
  
  $self->state(TASK_READY);
}

sub process_task_ready
{
  my $self = shift;

  if ($self->configurator()->max_portions() and
    $self->portions_executed() >= $self->configurator()->max_portions()) {
    die "Max number of portions exceeded";
  }
  warn "Fetching data portion.\n";
  $self->task()->fetch_portion()
    or die "Cannot fetch portion";

  $self->state(GOT_PORTION);
}

sub process_got_portion
{
  my $self = shift;

  warn "Processing portion.\n";
  $self->task()->process_portion()
    or die "Cannot process portion";

  $self->portions_executed($self->portions_executed + 1);;
  $self->state(PORTION_DONE);
}

sub process_portion_done
{
  my $self = shift;

  warn "Portion done, sending results.\n";
  $self->task()->send_result()
    or die "Cannot send result";
  warn "Results sent.\n";

  warn "Cleaning temporary files.\n";
  $self->task()->clean_temp_files();

  $self->state(TASK_READY);
}

1;
