package Configurator;

use strict;
use warnings;

use File::Spec;
use File::Temp qw(tempdir);
use Getopt::Long;

use NodeDesc;

use fields qw(config_file http_ua_string workdir create_workdir
              client_manager_urls nodedesc debug http_proxy add_arg
              impersonate max_portions ca_path key_file cert_file
             );

sub new
{
  my $self = shift;
  $self = fields::new($self) unless (ref $self);

  $self->set_defaults();
  $self->parse_config();

  return $self;
}

#
# Defaults
#

sub set_defaults
{
  my $self = shift;

  $self->{config_file} =
    File::Spec->catfile(File::Spec->curdir(), "gcli.conf.xml");
  $self->{http_ua_string} = "X-Com client/0.0";
  $self->{workdir} = undef;
  $self->{create_workdir} = 0;
  $self->{nodedesc} = NodeDesc->new;
  $self->{client_manager_urls} = [];
  $self->{debug} = 0;
  $self->{impersonate} = 0;
  $self->{max_portions} = 0;
  $self->{cert_file} = undef;
  $self->{key_file} = undef;
  $self->{ca_path} = undef;
}

#
# Parse config and command line parameters
#
sub parse_config
{
  my $self = shift;
  my $config_file;
  my @client_manager_urls;
  my $client_id;
  my $debug;
  my $tmpdir;
  my $http_proxy;
  my $add_arg;
  my $impersonate;
  my $max_portions;
  my $key_file;
  my $cert_file;
  my $ca_path;

  GetOptions(
    'f=s' => \$config_file,
    's=s' => \@client_manager_urls,
    'c=s' => \$client_id,
    'd' => \$debug,
    'T=s' => \$tmpdir,
    'http-proxy=s' => \$http_proxy,
    'add-arg=s' => \$add_arg,
    'impersonate' => \$impersonate,
    'max-portions=i' => \$max_portions,
    'key-file=s' => \$key_file,
    'cert-file=s' => \$cert_file,
    'ca-path=s' => \$ca_path
  );

  if($config_file) {
    $self->{config_file} = $config_file;
  }
  $self->parse_config_file();

  if(@client_manager_urls) {
    $self->{client_manager_urls} = \@client_manager_urls;
  }
  if($client_id) {
    $self->nodedesc()->client_id($client_id);
  }
  if($debug) {
    $self->{debug} = 1;
  }
  if($tmpdir) {
    $tmpdir =~ /^(.*)$/;
    $self->{workdir} = $1;
    $self->{create_workdir} = 1;
  }
  if($http_proxy) {
    $self->{http_proxy} = $http_proxy;
  }
  if($add_arg) {
    # Untainting add_arg, assume it is secure
    $add_arg =~ /^(.*)$/;
    $self->{add_arg} = $1;
  }
  if ($impersonate) {
    $self->{impersonate} = 1;
  }
  if ($max_portions) {
    $self->{max_portions} = $max_portions;
  }
  if (defined $ca_path) {
    $self->{ca_path} = $ca_path;
    warn "Using $ca_path as CA directory for HTTPS.";
  }
  if((defined $key_file) && (defined $cert_file) && (defined $ca_path)) {   
    $self->{cert_file} = $cert_file;
    $self->{key_file} = $key_file;
    warn "Using $key_file as private key and $cert_file as certificate for HTTPS.";
  }
  if (defined $ca_path) {
    unless (eval "require Crypt::SSLeay") {
      warn "Couldn't find Crypt::SSLeay. HTTPS is not supported.";
    }
  }
  if(not defined($self->workdir())) {
    if(not $self->debug()) {
      $self->{workdir} = tempdir('x-com-client-XXXX', TMPDIR => 1, CLEANUP => 1);
      # XXX - should be reconsidered fi this is really needed here.
      # It is for restart (bug #16) to work correctly.
      $self->{create_workdir} = 1;
    } else {
      $self->{workdir} = tempdir('x-com-client-XXXX', TMPDIR => 1);
      # XXX - should be reconsidered fi this is really needed here.
      # It is for restart (bug #16) to work correctly.
      $self->{create_workdir} = 1;
    }
  }
  if(@{$self->{client_manager_urls}} == 0) {
    $self->usage();
    die "Client manager URL is not defined";
  }
}

#
# Parse configuration file
#
sub parse_config_file
{
  my $self = shift;
  
  # Do noting for now
}

sub ua_string
{
  my $self = shift;

  return $self->{http_ua_string};
}

sub workdir
{
  my $self = shift;

  return $self->{workdir};
}

sub create_workdir
{
  my $self = shift;

  return $self->{create_workdir};
}

sub nodedesc
{
  my $self = shift;

  return $self->{nodedesc};
}

sub climan_urls
{
  my $self = shift;

  return @{$self->{client_manager_urls}};
}

sub debug
{
  my $self = shift;

  return $self->{debug};
}

sub http_proxy
{
  my $self = shift;
  return $self->{http_proxy};
}

sub add_arg
{
  my $self = shift;
  return $self->{add_arg};
}

sub impersonate
{
  my $self = shift;
  return $self->{impersonate};
}

sub max_portions
{
  return shift->{max_portions};
}

sub key_file
{
  return shift->{key_file};
}

sub cert_file
{
  return shift->{cert_file};
}

sub ca_path
{
  return shift->{ca_path};
}

sub new_executor
{
  my $self = shift;
  if ($self->impersonate()) {
    require Exec::Sudo;
    return Exec::Sudo->new();
  } else {
    require Exec::Plain;
    return Exec::Plain->new();
  }
  # NOTREACHED
  return undef;
}

sub usage
{
  print <<EOT
Usage: client.pl [-d] [-c <client id>] [-T <directory>] [--http-proxy <URL>] [--add-arg <argument>] -s <URL> [-ssl <ssl mode>] [--cert-file <certificate file>] [--key-file <key file>] [--ca-path <CA path>]

-d Enables debug output
-c Sets client id passed to server
-T Sets temporary directory to use
--http-proxy sets the URL to HTTP PROXY to connect to
-s Sets the URL to client manager to connect to.
--add-arg Sets additional argumetn passed to client part both in gcprepare and gctask.
--impersonate Task received from server MUST have 'api' attribute set to 'processes'
               and MUST have 'user' attribute set to user name to execute client part as.
--cert-file Full path to client certificate file for HTTPS.
--key-file Full path to client private key file fot HTTPS.
--ca-path Full path to CA directory for HTTPS.

EOT
}

1;
