#
# This module is inspired by Sean M. Burke's LWP::UserAgent::Determined module
#
package LWP::UserAgent::Retrying;

use strict;
use warnings;

use LWP::UserAgent;

our @ISA = qw(LWP::UserAgent);

#
# Hash keys in addition to those of LWP::UserAgent:
# sleeper - reference to object of Sleeper class)
# codes_to_retry - reference to list of HTTP codes to retry request
#
sub new
{
  my $class = shift;
  my %args = @_;
  my $sleeper = delete $args{sleeper} ||
    LWP::UserAgent::Retrying::Sleeper->new();
  my @codes_to_retry = @{delete $args{codes_to_retry} ||
    [
      '408', # Request Timeout
      '500', # Internal Server Error
      '502', # Bad Gateway
      '503', # Service Unavailable
      '504'  # Gateway Timeout
    ]
  };

  my $self = $class->SUPER::new(%args);
  $self->{sleeper} = $sleeper;
  $self->{codes_to_retry} = \@codes_to_retry;
  return bless ($self, $class);
}

sub simple_request
{
  my $self = shift;
  my $sleeper = $self->{sleeper};
  my $response;
  for ($sleeper->reset(); not $sleeper->finished(); $sleeper->sleep()) {
    $response = $self->SUPER::simple_request(@_);
    last until (grep($response->code() == $_, @{$self->{codes_to_retry}}));
  }
  return $response;
}

package LWP::UserAgent::Retrying::Sleeper;

use strict;
use warnings;

my %_default_args = (
  repeats    => 1,
  base_sleep => 0,
  rand_sleep => 1
);

use fields qw(repeats cur_repeat base_sleep rand_sleep);

sub new
{
  my $self = shift;
  $self = fields::new($self) unless (ref $self);
  my %args = (%_default_args, @_);
  %$self = %args;
  $self->reset();
  return $self;
}

sub repeats
{
  my $self = shift;
  if (@_) {$self->{repeats} = shift;}
  return $self->{repeats};
}

sub base_sleep
{
  my $self = shift;
  if (@_) {$self->{base_sleep} = shift;}
  return $self->{base_sleep};
}

sub rand_sleep
{
  my $self = shift;
  if (@_) {$self->{rand_sleep} = shift;}
  return $self->{rand_sleep};
}

sub reset
{
  my $self = shift;
  $self->{cur_repeat} = 0;
}

sub sleep
{
  my $self = shift;
  if ($self->finished()) {
    require Carp;
    Carp::carp(__PACKAGE__ . "::sleep called when finished() is true");
  }
  CORE::sleep($self->{base_sleep} + rand($self->{rand_sleep}));
  $self->{cur_repeat}++;
}

sub finished
{
  my $self = shift;
  return $self->{cur_repeat} >= $self->{repeats};
}

1;
