package Task;

use strict;
use warnings;

use File::Path;
use XML::Parser;

use Configurator;
use Util qw(untaint_filename);
use Task::File;
use Task::Portion;
use Task::Result;
use Task::ClientPart;

#
# Arguments:
# - Reference to Client owning Task
# - Reference to HTTP::Response with task descrption
#   OR reference to process-task XML element object
#   OR reference to ClientManagers object AND LWP::UserAgent object
#
sub new
{
  my $class = shift;
  my $client = shift;
  my $src = shift;

  my $self = {};
  bless($self, $class);

  $self->{CLIENT} = $client;
  $self->{FILES} = [];
  $self->{CLIPART} = undef;
  $self->{PORTION} = undef;
  $self->{RESULT} = undef;
  $self->{UA} = undef;
  $self->{API} = undef;
  $self->{USER} = undef;
  $self->{EXECUTOR} = undef;

  warn "Preparing working directory\n";
  $self->prepare_workdir();
  if ($src->isa("HTTP::Response")) {
    $self->parse_http_response($src);
  } elsif (ref($src) =~ /::process-task$/) {
    $self->parse_xml_object($src);
  } elsif ($src->isa("ClientManagers")) {
    $self->connect_climan($src, @_);
  } else {
    die "Not a HTTP::Response or XML process-task element or ClientManagers";
  }

  return $self;
}

sub connect_climan
{
  my $self = shift;
  my $climan = shift;
  my $ua = shift;

  my $req = Task::Request->new(
    $self->client()->configurator()->nodedesc(), 
    $climan->get_next_url()
  );

  my $resp = $ua->request($req);
  if(!$resp->is_success()) {
    die "Cannot connect to client manager";
  }

  $self->parse_http_response($resp);
  $self->ua($ua);
}

sub parse_http_response
{
  my $self = shift;
  my $resp = shift;

  if ($resp->isa("HTTP:Response")) {
    die "Not a HTTP::Response object";
  }

  if(!$resp->is_success()) {
    die "Response did not succeed, cannot create Task object";
  }
  
  my $task_desc_xml = $resp->decoded_content(raise_error => "true");

  my $parser = XML::Parser->new(Style => 'Objects', Pkg => 'Task::XML');
  my $task_desc_parsed = $parser->parse($task_desc_xml);

  # First element of the array must contain process-task XML element
  $self->parse_xml_object($task_desc_parsed->[0]);
}

sub parse_xml_object
{
  my $self = shift;
  my $task_elem = shift;

  ref($task_elem) =~ /::process-task$/
    or die "Not process-task XML element";

  $self->{TASK_ID} = $task_elem->{'task-id'} or die "Task id should be defined";
  $self->{SESSION_ID} = $task_elem->{'session-id'} or die "Session id should be defined";
  $self->{TASK_NAME} = $task_elem->{'task-name'} or die "Task name should be defined";
  if (not defined($task_elem->{'api'})) {
    $self->{API} = 'perl';
  } else {
    # Be liberal in what you accept
    $self->{API} = lc $task_elem->{'api'};
  }
  if ($self->api() ne 'perl' and $self->api() ne 'processes') {
    die "API attribute should be 'perl' (default) or 'processes'";
  }
  if (defined $task_elem->{'user'}) {
    $task_elem->{'user'} =~ /^([-[:alnum:]_.]*)$/;
    $self->{USER} = $1;
  }
  if ($self->client()->configurator()->impersonate()) {
    $self->api() eq 'processes'
      or die "Impersonation is possible only for task 'processes' API ";
    $self->user()
      or die "Task 'user' attribute should be defined to impersonate ";
  } else {
    $self->user()
      and die "Impersonation is not enabled but task 'user' attribute is defined ";
  }
  if ($self->api() eq 'processes') {
    $self->{EXECUTOR} = $self->client()->configurator()->new_executor();
    if ($self->user()) {
      $self->executor()->user($self->user());
    }
  }

  foreach my $elem (@{$task_elem->{Kids}}) {
    $self->process_elem($elem);
  }

  # Creating ClientPart object for 'perl' client API
  # XXX - think about reconsider
  if($self->api() eq 'perl') {
    $self->{CLIPART} = Task::ClientPart::Perl->new($self);
    if (defined($self->{PORTION})) {
      $self->{PORTION}->filename($self->clipart()->portion());
    }
    if (defined($self->{RESULT})) {
      $self->{RESULT}->filename($self->clipart()->result());
    }
  } 
}

sub process_elem
{
  my $self = shift;
  my $elem = shift;

  if (ref($elem) =~ /::file$/) {
    $self->process_file($elem);
  } elsif (ref($elem) =~ /::portion-server$/) {
    $self->process_portion_server($elem);
  } elsif (ref($elem) =~ /::result-server$/) {
    $self->process_result_server($elem);
  } elsif (ref($elem) =~ /::status-server$/) {
    $self->process_status_server($elem);
  } elsif (ref($elem) =~ /::portion-command$/) {
    $self->process_portion_command($elem)
  } elsif (ref($elem) =~ /::init-command$/) {
    $self->process_init_command($elem)
  } elsif (ref($elem) =~ /::Characters$/ and $elem->{Text} =~ /^\s*$/s) {
    # Skip empty text block - do notinhg
  } else {
    die "Unknown element among process-task children";
  }
}

sub process_file
{
  my $self = shift;
  my $elem = shift;

  ref($elem) =~ /::file$/ or die "Not a file element";

  push @{$self->{FILES}}, Task::File->new($self, $elem);
}

sub process_portion_server
{
  my $self = shift;
  my $elem = shift;

  ref($elem) =~ /::portion-server$/ or die "Not a portion-server element";

  $self->{PORTION} = Task::Portion->new($self, $elem);
  if (defined($self->clipart())) {
    $self->{PORTION}->filename($self->clipart()->portion());
  }
}

sub process_result_server
{
  my $self = shift;
  my $elem = shift;

  ref($elem) =~ /::result-server$/ or die "Not a result-server element";

  $self->{RESULT} = Task::Result->new($self, $elem);
  if (defined($self->clipart())) {
    $self->{RESULT}->filename($self->clipart()->result());
  }
}

sub process_status_server
{
  my $self = shift;
  my $elem = shift;

  ref($elem) =~ /::status-server$/ or die "Not a status-server element";
}

sub process_init_command
{
  my $self = shift;
  my $elem = shift;

  ref($elem) =~ /::init-command$/ or die "Not a init-command element";

  if ($self->api() ne 'processes') {
    die "Init-command element mey be present only for 'processes' API";
  }
  
  if (not defined $self->clipart()) {
    $self->{CLIPART} = Task::ClientPart::Processes->new($self, $elem);
  } else {
    $self->clipart()->fill_init_command($elem);
  }
}

sub process_portion_command
{
  my $self = shift;
  my $elem = shift;

  ref($elem) =~ /::portion-command$/ or die "Not a portion-command element";

  if ($self->api() ne 'processes') {
    die "Portion-command element mey be present only for 'processes' API";
  }

  if (not defined $self->clipart()) {
    $self->{CLIPART} = Task::ClientPart::Processes->new($self, $elem);
  } else {
    $self->clipart()->fill_portion_command($elem);
  }

  if (defined($self->{PORTION})) {
    $self->{PORTION}->filename($self->clipart()->portion());
  }
  if (defined($self->{RESULT})) {
    $self->{RESULT}->filename($self->clipart()->result());
  }
}

sub client
{
  my $self = shift;
  return $self->{CLIENT};
}

sub files
{
  my $self = shift;
  return $self->{FILES};
}

sub ua
{
  my $self = shift;
  if(@_) {$self->{UA} = shift;}
  return $self->{UA};
}

sub portion
{
  my $self = shift;
  return $self->{PORTION};
}

sub result
{
  my $self = shift;
  return $self->{RESULT};
}

sub clipart
{
  my $self = shift;
  return $self->{CLIPART};
}

sub task_name
{
  my $self = shift;
  return $self->{TASK_NAME};
}

sub task_id
{
  my $self = shift;
  return $self->{TASK_ID};
}

sub session_id
{
  my $self = shift;
  return $self->{SESSION_ID};
}

sub api
{
  my $self = shift;
  return $self->{API};
}

sub user
{
  my $self = shift;
  return $self->{USER};
}

sub executor
{
  my $self = shift;
  return $self->{EXECUTOR};
}

sub prepare_workdir
{
  my $self = shift;

  my $wd = $self->client()->configurator()->workdir();
  my $wd_mask = 0700;
  if (-d $wd) {
    if (not(-r _ && -w _)) {
      die "Working directory $wd should be both readable and writable";
    }; 
  } elsif ($self->client()->configurator()->create_workdir) {
    mkpath( $wd, {mask => $wd_mask}); 
  } else {
    die "Working directory $wd does not exist";
  }
}

sub clean_workdir
{
  my $self = shift;

  my $wd = $self->client()->configurator()->workdir();
  foreach my $file (@{$self->files()}) {
    $file->unlink_in_dir($wd);
  }
  $self->clean_temp_files();
}

sub clean_temp_files
{
  my $self = shift;

  my $wd = $self->client()->configurator()->workdir();
  $self->portion()->unlink();
  $self->result()->unlink();
}

sub fetch_files
{
  my $self = shift;

  foreach my $file (@{$self->files()}) {
    $file->fetch_into_dir($self->client()->configurator()->workdir());
  }
}

sub init_clipart
{
  my $self = shift;

  chdir $self->client->configurator()->workdir()
    or die "Cannot chdir to working directory " .
             $self->client->configurator()->workdir();
  return $self->clipart()->init();
}

sub fetch_portion
{
  my $self = shift;
  $self->portion()->fetch();
}

sub process_portion
{
  my $self = shift;

  chdir $self->client->configurator()->workdir()
    or die "Cannot chdir to working directory " .
             $self->client->configurator()->workdir();
  return $self->clipart()->execute();
}

sub send_result
{
  my $self = shift;
  $self->result()->post_from_dir($self->client()->configurator()->workdir());  
}

package Task::Request;

use strict;
use warnings;

use HTTP::Request;
our @ISA = qw(HTTP::Request);

#
# Arguments:
# - NodeDesc object to get client data.
# - URL to client manager to make request to.
#
sub new
{
  my $class = shift;
  my $nodedesc = shift;
  my $url = shift;

  my $self = $class->SUPER::new();
  $self->header('Content-Type' => 'text/xml');

  my $content;
  my $writer = XML::Writer->new(OUTPUT => \$content);
  $writer->startTag('get-task');
  $nodedesc->write_xml($writer);
  $writer->endTag('get-task');

  $self->content($content);
# Get is for Abyss Web Server
  $self->method('POST');
#  $self->method('GET');
  $self->uri($url);

  bless ($self, $class);
  return $self;
}

1;
