package Task::ClientPart::Processes;

use strict;
use warnings;

use File::Spec;

use Util qw(untaint_filename);

use first qw(Time::HiRes CORE time);

use fields qw(TASK PORTION RESULT INIT_COMMAND PORTION_COMMAND WORK_TIME);

sub new
{
  my $self = shift;
  $self = fields::new($self) unless (ref $self);

  my $task = shift;
  my $elem = shift;

  $self->{TASK} = $task;

  if (ref($elem) =~ /::portion-command$/) {
    $self->fill_portion_command($elem);
  } elsif (ref($elem) =~ /::init-command$/) {
    $self->fill_init_command($elem);
  } else {
    die "Not a portion-command or init-command element";
  }

  warn "Cannot import Time::HiRes::time(), client part work time will be measured with reduced accuracy"
    if (first->used_from('time') eq 'CORE::time');

  return $self;
}

sub fill_portion_command
{
  my $self = shift;
  my $elem = shift;

  ref($elem) =~ /::portion-command$/
    or die "Not a portion-command element";

  $self->{PORTION} = untaint_filename($elem->{portion})
    or die "Portion file name is not defined";
  $self->{RESULT} = untaint_filename($elem->{result})
    or die "Result file name is not defined";

  scalar(@{$elem->{Kids}}) == 1
    or die "Invalid number of child elements";

  ref($elem->{Kids}->[0]) =~ /::Characters$/
    or die "The kid of portion-command is not text";

  my $cl = $elem->{Kids}->[0]->{Text};
  # Stripping head and tail space characters
  $cl =~ s/^[[:space:]]*([^[:space:]].*[^[:space:]])[[:space:]]*$/$1/;
  # Maybe untaint_filename is too strict here since it doesn't allow slashes and shell metacharacters
  $self->{PORTION_COMMAND} = untaint_filename($cl);
}

sub fill_init_command
{
  my $self = shift;
  my $elem = shift;

  ref($elem) =~ /::init-command$/
    or die "Not a init-command element";

  scalar(@{$elem->{Kids}}) == 1
    or die "Invalid number of child elements";

  ref($elem->{Kids}->[0]) =~ /::Characters$/
    or die "The kid of portion-command is not text";

  my $cl = $elem->{Kids}->[0]->{Text};
  # Stripping head and tail space characters
  $cl =~ s/^[[:space:]]*([^[:space:]].*[^[:space:]])[[:space:]]*$/$1/;
  # Maybe untaint_filename is too strict here since it doesn't allow slashes and shell metacharacters
  $self->{INIT_COMMAND} = untaint_filename($cl);
}

sub task
{
  my $self = shift;
  return $self->{TASK};
}

sub portion
{
  my $self = shift;
  return $self->{PORTION};
}

sub result
{
  my $self = shift;
  return $self->{RESULT};
}

sub init_command
{
  my $self = shift;
  return $self->{INIT_COMMAND};
}

sub portion_command
{
  my $self = shift;
  return $self->{PORTION_COMMAND};
}

sub work_time
{
  my $self = shift;
  return $self->{WORK_TIME};
}

sub init
{
  my $self = shift;

  local @ENV{qw(IFS CDPATH ENV BASH_ENV PATH)};
  delete @ENV{qw(IFS CDPATH ENV BASH_ENV PATH)};

  my $cl = $self->init_command();
  my @cl = split(' ', $cl);
  $cl[0] = File::Spec->catfile(File::Spec->curdir(), $cl[0]);
  if (defined $self->task()->client()->configurator()->add_arg()) {
    push @cl, $self->task()->client()->configurator()->add_arg();
  }
  my $rc = $self->task()->executor()->system(@cl);
  if ($rc == -1) {
    die "Cannot execute $cl: $!";
  } elsif ($rc & 127) {
    die sprintf("%s was killed by signal %d", $cl, $rc & 127);
  } elsif ($rc >> 8) {
    die sprintf("%s finished unsuccessfully with exit status %d", $cl, $rc >> 8);
  }
  return $rc == 0;  
}

sub execute
{
  my $self = shift;

  local @ENV{qw(IFS CDPATH ENV BASH_ENV PATH)};
  delete @ENV{qw(IFS CDPATH ENV BASH_ENV PATH)};

  my $cl = $self->portion_command();
  my @cl = split(' ', $cl);
  $cl[0] = File::Spec->catfile(File::Spec->curdir(), $cl[0]);
  my $time_before = time();
  if (defined $self->task()->client()->configurator()->add_arg()) {
    push @cl, $self->task()->client()->configurator()->add_arg();
  }
  $cl = join(' ', @cl);
  my $rc = $self->task()->executor()->system(@cl);
  if ($rc == -1) {
    die "Cannot execute $cl: $!";
  } elsif ($rc & 127) {
    die sprintf("%s was killed by signal %d", $cl, $rc & 127);
  } elsif ($rc >> 8) {
    die sprintf("%s finished unsuccessfully with exit status %d", $cl, $rc >> 8);
  }
  my $time_after = time();
  $self->{WORK_TIME} = $time_after - $time_before;
  return $rc == 0;  
}

package Task::ClientPart::Perl;

use strict;
use warnings;

use File::Spec;
use File::Temp qw(mktemp);
use Opcode;
use Safe;

use Util qw(untaint_filename);

use first qw(Time::HiRes CORE time);

use fields qw(TASK PORTION RESULT SAFE WORK_TIME);

sub new
{
  my $self = shift;
  $self = fields::new($self) unless (ref $self);

  my $task = shift;

  $self->{TASK} = $task;    
  $self->{PORTION} =
    mktemp(File::Spec->catfile($task->client()->configurator()->workdir(), 'prtXXXX'));
  $self->{RESULT} =
    mktemp(File::Spec->catfile($task->client()->configurator()->workdir(), 'resXXXX'));
  $self->{SAFE} = Safe->new();
  $self->safe()->permit(Opcode::opset_to_ops(Opcode::full_opset));
  my @additional_share = qw(
    &UNIVERSAL::isa
    &UNIVERSAL::can
    &UNIVERSAL::VERSION
    &DynaLoader::boot_DynaLoader
  );
  $self->safe()->share_from('main', \@additional_share)
    or warn $@;

  warn "Cannot import Time::HiRes::time(), client part work time will be measured with reduced accuracy"
    if (first->used_from('time') eq 'CORE::time');

  return $self;
}

sub task
{
  my $self = shift;
  return $self->{TASK};
}

sub portion
{
  my $self = shift;
  return $self->{PORTION};
}

sub result
{
  my $self = shift;
  return $self->{RESULT};
}

sub safe
{
  my $self = shift;
  return $self->{SAFE};
}

sub work_time
{
  my $self = shift;
  return $self->{WORK_TIME};
}

sub init
{
  my $self = shift;

  local @ENV{qw(IFS CDPATH ENV BASH_ENV PATH)};
  delete @ENV{qw(IFS CDPATH ENV BASH_ENV PATH)};

  my $gctask =
    File::Spec->catfile(File::Spec->curdir(), "gctask");
  # This is for rdo to work in taint mode on MSWin32
  # if is qiock and DIRTY hack to make it work on Linux (Linux's Safe->rdo in taint mode
  # does not like /tmp permissions (1777) and denies use of its subdirs.
  # XXX - MUST be reconsidered.
  if ($^O eq 'MSWin32') {
    $gctask = File::Spec->rel2abs($gctask);
  }

  -r $gctask
    or die "Cannot find file $gctask";

  $self->safe()->rdo($gctask)
    or die "Cannot rdo file $gctask: $@";
  my @arguments = ($self->task()->task_id());
  if (defined $self->task()->client()->configurator()->add_arg()) {
    push @arguments, $self->task()->client()->configurator()->add_arg();
  }
  # We should untaint $arg_str beacause tainted data cannot be passed to eval
  # XXX - maybe data should be checked (and untainted) somewhere earlier
  join(',', map('"' . quotemeta($_) . '"', @arguments)) =~ /^(.*)$/;
  my $arg_string = $1;
  $self->safe()->reval("gcprepare($arg_string);")
    or die "Cannot call gcprepare subroutine: $@";
}

sub execute
{
  my $self = shift;

  local @ENV{qw(IFS CDPATH ENV BASH_ENV PATH)};
  delete @ENV{qw(IFS CDPATH ENV BASH_ENV PATH)};

  my $task_id = $self->task()->task_id();
  my $portion_num = $self->task()->portion()->cur_num();
  my $portion_taskarg = $self->task()->portion()->cur_taskarg();
  # Untainting $task_id
  $task_id =~ /^(.*)$/;
  $task_id = $1;
  my $portion = $self->portion();
  my $result = $self->result();
  my @arguments = ($task_id, $portion_taskarg, $portion_num, $portion, $result);
  if (defined $self->task()->client()->configurator()->add_arg()) {
    push @arguments, $self->task()->client()->configurator()->add_arg();
  }
  my $arg_string = join(',', map('"' . quotemeta($_) . '"', @arguments));
  my $eval_exp = "gctask($arg_string);";
  my $time_before = time();
  my $rc = $self->safe()->reval($eval_exp)
    or die "Cannot call gctask subroutine: $@";
  my $time_after = time();
  $self->{WORK_TIME} = $time_after - $time_before;
  return $rc;
}

1;
