package Task::File;

use strict;
use warnings;

use Archive::Tar;
use File::Spec;
use IO::File;
use URI;

use Util qw(untaint_filename);

use URLList;
our @ISA = qw(URLList);

sub new
{
  my $class = shift;
  my $task = shift;
  my $elem = shift;

  my $self = $class->parse_xml_object($elem);
  $self->{TASK} = $task;

  bless($self, $class);
  return $self;
}

#
# Creates Task::File from XML::Parser object style object
#
sub parse_xml_object
{
  my $class = shift;
  my $elem = shift;

  ref($elem) =~ /::file$/ or die "Not file XML element";

  my $self = $class->SUPER::parse_xml_object($elem);

  if(lc $elem->{executable} eq 'true' or lc $elem->{executable} eq 'false') {
    $self->{EXECUTABLE} = lc $elem->{executable} eq 'true';
  } else {
    die "'executable' attribute should be 'true' or 'false'";
  }
  if(defined $elem->{filename}) {
    $self->{NAME} = untaint_filename($elem->{filename});
  } else {
    die "'filename' attribute should be defined";
  }
  if(defined $elem->{id}) {
    $self->{ID} = $elem->{id};
  } else {
    die "'id' attribute should be defined";
  }
  if(defined $elem->{archive} and lc $elem->{archive} eq 'true') {
    die "Archive file should not be executable"
      if $self->executable();
    $self->{ARCHIVE} = 1;
  } else {
    $self->{ARCHIVE} = 0;
  }

  bless($self, $class);
  return $self;
}

sub executable
{
  my $self = shift;
  if(@_) {$self->{EXECUTABLE} = shift;}
  return $self->{EXECUTABLE}; 
}

sub archive
{
  my $self = shift;
  if(@_) {$self->{ARCHIVE} = shift;}
  die "Archive file should not be executable"
    if $self->executable();
  return $self->{ARCHIVE};
}

sub name
{
  my $self = shift;
  if(@_) {
    my $name = shift;
    $self->{NAME} = untaint_filename($name)
  }
  return $self->{NAME}; 
}

sub id
{
  my $self = shift;
  if(@_) {$self->{ID} = shift;}
  return $self->{ID}; 
}

sub task
{
  my $self = shift;
  return $self->{TASK};
}

#
# Unlinks file using name from the description in the specified
# diretory
#
sub unlink_in_dir
{
  my $self = shift;
  my $dir = shift;
  
  my $fname = 
    File::Spec->catfile($dir, $self->name());

  unlink($fname);
}

#
# Fethches file from URL
# Arguments are:
# - IO::File: File handle to place file in
#
sub do_fetch
{
  my $self = shift;
  my $file = shift or die "File handle is not defined";

  my $uri = URI->new($self->get_next_url());
  $uri->query_form(
    'client-id' => $self->task()->client()->configurator()->nodedesc()->client_id()
  );
  # Work around bug in old LWP versions
  # This bug make taint check fail in connect
  $uri->host();
  my $req = HTTP::Request->new(GET => $uri);
  my $resp = $self->task()->ua()->request($req);
  $resp->is_success or die "Cannot fetch file from " . $req->uri();

  my $file_content = $resp->decoded_content();
  $file->write($file_content, length $file_content);
}

#
# Fetches file to the specified directory (using name from file description)
# Arguments are:
# - Directory name to place file into.
#
sub fetch_into_dir
{
  my $self = shift;
  my $dir = shift
    or die "Directory name is not defined";

  use Scalar::Util qw(tainted);
  my $perms;
  if($self->executable()) {
    $perms = 0500;
  } else {
    $perms = 0400;
  }
 
  $self->unlink_in_dir($dir);

  my $fname =
    File::Spec->catfile($dir, $self->name());
  my $fh = IO::File->new($fname, O_CREAT | O_WRONLY, $perms)
    or die "Cannot open file";
  $fh->binmode();

  $self->do_fetch($fh);
  $fh->close();
# If the file is archived, unpack it.
  if($self->archive()) {
    my $tar = Archive::Tar->new( )
      or die "Cannot create Archive::Tar object";
    $tar->setcwd($dir);
    $tar->read($fname, 0, { extract => 1 })
      or die "Cannot unpack archive file " . $self->name();

    unlink($fname);
  }
}

1;
